@@ -124,52 +124,93 @@ VersionNumber* simonMaxVersion, const QString& license, QList<Author*> authors)
 
 bool Scenario::setupChildScenarios()
 {
-    m_childScenarios.clear();
+  m_childScenarios.clear();
 
-    foreach (const QString& id, m_childScenarioIds)
+  foreach (const QString& id, m_childScenarioIds)
+  {
+    Scenario* child = ScenarioManager::getInstance()->getScenario(id);
+
+    if (child)
     {
-        Scenario* child = ScenarioManager::getInstance()->getScenario(id);
+      m_childScenarios << child;
+      child->setParentScenario(this);
 
-        if (child)
-        {
-            m_childScenarios << child;
-            child->setParentScenario(this);
-
-            kDebug() << child->id() + " is set as child of " + this->id();
-        }
-        else
-        {
-            kDebug() << "Error: Child id '" + id + "'' is unavailable";
-        }
+      kDebug() << child->id() + " is set as child of " + this->id();
     }
+    else
+    {
+      kDebug() << "Error: Child id '" + id + "'' is unavailable";
+    }
+  }
 
-    return true;
+  return true;
 }
 
 QList<Scenario*> Scenario::childScenarios() const
 {
-    return m_childScenarios;
+  return m_childScenarios;
 }
 
 QStringList Scenario::childScenarioIds() const
 {
-    return m_childScenarioIds;
+  return m_childScenarioIds;
 }
 
 Scenario* Scenario::parentScenario()
 {
-    return m_parentScenario;
+  return m_parentScenario;
 }
 
 void Scenario::setParentScenario(Scenario *parent)
 {
-    m_parentScenario = parent;
-    updateActivation();
+  m_parentScenario = parent;
+  updateActivation();
 }
 
 void Scenario::setChildScenarioIds(QStringList ids)
 {
-    m_childScenarioIds = ids;
+  m_childScenarioIds = ids;
+}
+
+bool Scenario::updateChildScenarioIds(const QString& id, const QStringList& ids)
+{
+  QString path = Scenario::pathFromId(id, QString(), false);
+  kDebug() << "Updating child ids of " << path << " to " << ids;
+  QDomDocument doc("scenario");
+  QFile file(path);
+  if (!file.open(QIODevice::ReadOnly) || (!doc.setContent(&file))) {
+    return false;
+  }
+  QDomElement docElem = doc.documentElement();
+  docElem.setAttribute("lastModified", utcTime().toString(Qt::ISODate));
+  QDomElement childrenElem = docElem.firstChildElement("childscenarioids");
+  if (childrenElem.isNull()) {
+    kDebug() << "No child scenario id list element!";
+    childrenElem = doc.createElement("childscenarioids");
+    docElem.appendChild(childrenElem);
+  } else {
+    QDomElement idElem;
+    while (!(idElem = childrenElem.firstChildElement("scenarioid")).isNull())
+      childrenElem.removeChild(idElem); // remove all children
+  }
+
+  foreach(const QString& id, ids)
+  {
+    QDomElement idElem = doc.createElement("scenarioid");
+    idElem.appendChild(doc.createTextNode(id));
+    childrenElem.appendChild(idElem);
+  }
+
+  file.close();
+  file.setFileName(Scenario::pathFromId(id));
+  if (!file.open(QIODevice::WriteOnly|QIODevice::Truncate))
+    return false;
+
+  QString out = Scenario::pathFromId(id, QString(), false);
+  QByteArray data = doc.toString().toUtf8();
+  file.write(data);
+  file.close();
+  return true;
 }
 
 bool Scenario::update(const QString& name, const QString& iconSrc, int version, VersionNumber* simonMinVersion,
@@ -448,7 +489,7 @@ bool Scenario::readChildScenarioIds(QString path, QDomDocument* doc, bool delete
   {
     m_childScenarioIds.push_back(idElem.text());
 
-      kDebug() << "Child scenario id added: " + idElem.text();
+    kDebug() << "Child scenario id added: " + idElem.text();
 
     idElem = idElem.nextSiblingElement("scenarioid");
   }
@@ -458,7 +499,7 @@ bool Scenario::readChildScenarioIds(QString path, QDomDocument* doc, bool delete
   return true;
 }
 
-QStringList Scenario::explode(const QString& inFile)
+QStringList Scenario::explode(const QString& inFile, bool keepFile)
 {
   QFile f(inFile);
 
@@ -506,17 +547,24 @@ QStringList Scenario::explode(const QString& inFile)
   }
   //strip children
   while (!(childElem = childrenElem.firstChildElement("scenario")).isNull()) {
-    QDomElement idElem = d.createElement("scenarioid");
-    idElem.appendChild(d.createTextNode(childElem.attribute("name")));
-    childrenElem.appendChild(idElem); // add link
     childrenElem.removeChild(childElem);
   }
-  foreach (QString child, childrenSrc) //explode recursively
-    allScenarios << Scenario::explode(pathFromId(child));
+  foreach (QString child, childrenSrc) {//explode recursively
+    allScenarios << Scenario::explode(pathFromId(child), true);
+    QDomElement idElem = d.createElement("scenarioid");
+    idElem.appendChild(d.createTextNode(child));
+    childrenElem.appendChild(idElem); // add link
+  }
 
-  QString outerName = scenarioElem.attribute("name");
-  QString outerId = Scenario::createId(outerName);
-  QString outerPath = pathFromId(outerId);
+  QString outerId;
+  QString outerPath;
+  if (!keepFile) {
+    outerId = Scenario::createId(scenarioElem.attribute("name"));
+    outerPath = pathFromId(outerId);
+  } else {
+    outerId = idFromPath(inFile);
+    outerPath = inFile;
+  }
 
   QFile::remove(inFile);
   kDebug() << "Output path: " << outerPath;
@@ -526,19 +574,29 @@ QStringList Scenario::explode(const QString& inFile)
   } else {
     fInSrc.write(d.toString().toUtf8());
     fInSrc.close();
-    allScenarios << outerId;
+    allScenarios.insert(0, outerId);
   }
   return allScenarios;
 }
 
 
-QString Scenario::pathFromId(const QString& id, const QString& prefix)
+QString Scenario::pathFromId(const QString& id, const QString& prefix, bool local)
 {
-  if (prefix.isNull()) {
-    return KStandardDirs::locateLocal("appdata", "scenarios/"+id);
+  if (local) {
+    if (prefix.isNull())
+      return KStandardDirs::locateLocal("appdata", "scenarios/"+id);
+    else
+      return KStandardDirs::locateLocal("data", prefix+"scenarios/"+id);
+  } else {
+    if (prefix.isNull())
+      return KStandardDirs::locate("appdata", "scenarios/"+id);
+    else
+      return KStandardDirs::locate("data", prefix+"scenarios/"+id);
   }
-
-  return KStandardDirs::locateLocal("data", prefix+"scenarios/"+id);
+}
+QString Scenario::idFromPath(const QString& path)
+{
+  return QFileInfo(path).fileName();
 }
 
 QString Scenario::preferredPath() const
@@ -1041,39 +1099,39 @@ void Scenario::setListInterfaceCommands(QHash<CommandListElements::Element, Voic
 
 void Scenario::updateActivation()
 {
-    if (!m_parentScenario)
+  if (!m_parentScenario)
+  {
+    if (m_compoundCondition->isSatisfied())
     {
-        if (m_compoundCondition->isSatisfied())
-        {
-            kDebug() << "Scenario '" + m_name + "' and its applicable children should activate due to context!";
-            m_active = true;
-        }
-        else
-        {
-            kDebug() << "Scenario '" + m_name + "' and its children should deactivate due to context!";
-            m_active = false;
-        }
+      kDebug() << "Scenario '" + m_name + "' and its applicable children should activate due to context!";
+      m_active = true;
     }
     else
     {
-        if (m_compoundCondition->isSatisfied() && m_parentScenario->isActive())
-        {
-            kDebug() << "Scenario '" + m_name + "' and its applicable children should activate due to context!";
-            m_active = true;
-        }
-        else
-        {
-            kDebug() << "Scenario '" + m_name + "' and its children should deactivate due to context!";
-            m_active = false;
-        }
+      kDebug() << "Scenario '" + m_name + "' and its children should deactivate due to context!";
+      m_active = false;
     }
-
-    foreach(Scenario *s, m_childScenarios)
+  }
+  else
+  {
+    if (m_compoundCondition->isSatisfied() && m_parentScenario->isActive())
     {
-        s->updateActivation();
+      kDebug() << "Scenario '" + m_name + "' and its applicable children should activate due to context!";
+      m_active = true;
     }
+    else
+    {
+      kDebug() << "Scenario '" + m_name + "' and its children should deactivate due to context!";
+      m_active = false;
+    }
+  }
 
-    emit activationChanged();
+  foreach(Scenario *s, m_childScenarios)
+  {
+    s->updateActivation();
+  }
+
+  emit activationChanged();
 }
 
 
