@@ -177,11 +177,21 @@ QStringList ScenarioManagementWidget::importScenario(const QString& path, QTreeW
   }
 
   bool failed = false;
+  QHash<QString /*id*/, QTreeWidgetItem* /*prospective parent*/> parents;
+  QTreeWidgetItem *cur;
+  QStringList children;
   foreach (const QString& e, exploded) {
-    if (!displayScenario(e, widget)) {
+    if (!(cur = displayScenario(e, widget, &children))) {
       failed = true;
       break;
     }
+    if (parents.contains(e)) {
+      widget->takeTopLevelItem(widget->invisibleRootItem()->indexOfChild(cur));
+      parents[e]->addChild(cur);
+    }
+    foreach (const QString& child, children) {
+      parents.insert(child, cur);
+    }
   }
   if (failed) {
     foreach (const QString& e, exploded) {
@@ -197,11 +207,12 @@ void ScenarioManagementWidget::exportScenarioGHNS()
 {
   Scenario *s = getCurrentlySelectedScenario();
   if (!s) return;
+  if (!save()) return;
 
   QString path;
   if (askExportFull(s)) {
     //we need to merge this
-    path = KStandardDirs::locate("tmp", m_dataPrefix+"mergedscenario");
+    path = KStandardDirs::locateLocal("tmp", m_dataPrefix+"mergedscenario");
 
     if (!s->init()) {
       KMessageBox::sorry(this, i18n("Could not load scenario."));
@@ -235,6 +246,7 @@ void ScenarioManagementWidget::exportScenarioFile()
 {
   Scenario *s = getCurrentlySelectedScenario();
   if (!s) return;
+  if (!save()) return;
 
   if (!s->init()) {
     KMessageBox::sorry(this, i18n("Could not load scenario."));
@@ -337,13 +349,18 @@ void ScenarioManagementWidget::deleteScenario(const QString& id, bool removeFile
   kDebug() << "Widget: " << widget;
 
   //removing from list
-  if (item->parent() == 0) {
-    widget->invisibleRootItem()->removeChild(item);
-    kDebug() << "remove from widget";
-  } else {
-    kDebug() << "Remove from child";
-    item->parent()->removeChild(item);
-  }
+  QList<QTreeWidgetItem*> children = item->takeChildren();
+  QTreeWidgetItem *parent;
+  int i;
+  if (item->parent() == 0)
+    parent = widget->invisibleRootItem();
+  else
+    parent = item->parent();
+
+  i = parent->indexOfChild(item);
+  foreach (QTreeWidgetItem* child, children)
+    parent->insertChild(i++, child);
+  parent->removeChild(item);
 
   if (removeFile)
     idsToDelete << id;
@@ -440,7 +457,7 @@ void ScenarioManagementWidget::slotRemoved()
 }
 
 
-QTreeWidgetItem* ScenarioManagementWidget::displayScenario(const QString& scenario, QTreeWidget* widget)
+QTreeWidgetItem* ScenarioManagementWidget::displayScenario(const QString& scenario, QTreeWidget* widget, QStringList* children)
 {
     kDebug() << "Displaying scenario: " << scenario;
     Scenario *s = new Scenario(scenario);
@@ -451,6 +468,8 @@ QTreeWidgetItem* ScenarioManagementWidget::displayScenario(const QString& scenar
       return 0;
     }
     QTreeWidgetItem *ret = displayScenario(s, widget);
+    if (children)
+      *children = s->childScenarioIds();
     s->deleteLater();
     return ret;
 }
@@ -510,6 +529,13 @@ void ScenarioManagementWidget::initDisplay()
   QStringList  selectedIds = cg.readEntry("SelectedScenarios", QStringList() << "general");
 
   QStringList scenarioIds = ScenarioManager::getInstance()->getAllAvailableScenarioIds(m_dataPrefix);
+
+  //reorder scenarioIds according to selectedIds
+  for (int i = 0; i < selectedIds.count(); i++) {
+    scenarioIds.removeAll(selectedIds[i]);
+    scenarioIds.insert(i, selectedIds[i]);
+  }
+
   kDebug() << "Found scenarios: " << scenarioIds;
 
   QList<Scenario*> scenarios;
@@ -517,15 +543,11 @@ void ScenarioManagementWidget::initDisplay()
 
   foreach (const QString& id, scenarioIds) {
 
-    Scenario *s = new Scenario(id, m_dataPrefix);
-    if (!s->skim())
-    {
-      KMessageBox::information(this, i18nc("%1 is scenario id", "Could not init scenario \"%1\"", id));
-    }
-    else
-    {
-        scenarios << s;
-    }
+	  Scenario *s = new Scenario(id, m_dataPrefix);
+	  if (!s->skim())
+		  KMessageBox::information(this, i18nc("%1 is scenario id", "Could not init scenario \"%1\"", id));
+	  else
+		  scenarios << s;
   }
 
   //display the scenarios
@@ -571,27 +593,21 @@ void ScenarioManagementWidget::selectedScenarioSelected()
 
 void ScenarioManagementWidget::saveChildConfiguration(QTreeWidgetItem *parentItem)
 {
-    for (int i=0; i<parentItem->childCount(); i++)
-    {
-        //setup the item
-        QTreeWidgetItem *item = parentItem->child(i);
+  for (int i=0; i<parentItem->childCount(); i++)
+  {
+    //setup the item
+    QTreeWidgetItem *item = parentItem->child(i);
 
-        kDebug() << "Configuring children of: " << item->data(0, Qt::UserRole).toString();
+    kDebug() << "Configuring children of: " << item->data(0, Qt::UserRole).toString();
 
-        //save the item's child configuration
-        QStringList ids = getChildScenarioIds(item);
-        Scenario *s = new Scenario(item->data(0, Qt::UserRole).toString());
-        if (s->init())
-        {
-            s->setChildScenarioIds(ids);
-            s->save();
-        }
+    //save the item's child configuration
+    QStringList ids = getChildScenarioIds(item);
+    if (!Scenario::updateChildScenarioIds(item->data(0, Qt::UserRole).toString(), ids))
+      kWarning() << "Failed to update cihld scenario configuration";
 
-        ids.clear();
-
-        //configure item's children's children
-        saveChildConfiguration(item);
-    }
+    //configure item's children's children
+    saveChildConfiguration(item);
+  }
 }
 
 bool ScenarioManagementWidget::save()
