@@ -23,7 +23,7 @@
 
 #include<KLocalizedString>
 #include <QDir>
-
+#include <QSet>
 
 ModelCompilationAdapterSPHINX::ModelCompilationAdapterSPHINX(const QString &userName, QObject *parent)
   : ModelCompilationAdapter(userName, parent)
@@ -49,7 +49,7 @@ bool ModelCompilationAdapterSPHINX::startAdaption(AdaptionType adaptionType, con
   if (args.value("stripContext") == "true")
   {
     //remove context additions for prompts file
-    if(!removeContextAdditions())
+    if(!removeContextAdditions(adaptionType))
       return false;
   }
 
@@ -66,10 +66,11 @@ bool ModelCompilationAdapterSPHINX::startAdaption(AdaptionType adaptionType, con
   //merging scenarios
   mergeInputData(scenarioPathsIn, mergedVocabulary, mergedGrammar);
 
-  if(mergedVocabulary->empty())
+  if((adaptionType & ModelCompilationAdapter::AdaptLanguageModel) &&  mergedVocabulary->empty())
   {
     kDebug()<<"Empty vocabulary aborting adaptation";
-    abort();
+    emit adaptionAborted(ModelCompilation::InsufficientInput);
+    return false;
   }
 
   ADAPT_CHECKPOINT;
@@ -103,11 +104,6 @@ bool ModelCompilationAdapterSPHINX::storeModel(AdaptionType adaptionType, const
 
   ADAPT_CHECKPOINT;
 
-  if (!(adaptionType & ModelCompilationAdapter::AdaptLanguageModel)) // I do not fully understand the meaning of this code snippet, so I just copied it:(
-    return true;
-
-  ADAPT_CHECKPOINT;
-
   FileUtils::removeDirRecursive(workingDirPath+"/"+mName);
 
   //Creating a directory hierarchy, where model compilation will be executed
@@ -121,8 +117,8 @@ bool ModelCompilationAdapterSPHINX::storeModel(AdaptionType adaptionType, const
   QString fetc = workingDirPath+"/"+mName+"/etc/"+mName;
 
   kDebug()<<"Store dictionary";
-
-  if(!storeDictionary(adaptionType, fetc+DICT_EXT, trainedVocabulary, definedVocabulary,
+  if(!purgeUnusedVocabulary(vocabulary, grammar) ||
+     !storeDictionary(adaptionType, fetc+DICT_EXT, trainedVocabulary, definedVocabulary,
                       vocabulary))
   {
     emit error(i18n("Failed to store dictionary"));
@@ -141,7 +137,7 @@ bool ModelCompilationAdapterSPHINX::storeModel(AdaptionType adaptionType, const
   ADAPT_CHECKPOINT;
 
   kDebug()<<"Store phonelist";
-  if(!storePhonesList(adaptionType, fetc+PHONE_EXT, vocabulary, trainedVocabulary))
+  if(!(adaptionType == ModelCompilationAdapter::AdaptLanguageModel) && !storePhonesList(adaptionType, fetc+PHONE_EXT, vocabulary, trainedVocabulary))
   {
     emit error(i18n("Failed to store phones"));
     return false;
@@ -150,9 +146,11 @@ bool ModelCompilationAdapterSPHINX::storeModel(AdaptionType adaptionType, const
   ADAPT_CHECKPOINT;
 
   kDebug()<<"Store transcription & fields";
-  if(!storeTranscriptionAndFields(adaptionType, promptsPathIn, fetc+TRAIN_TRANSCRIPTION, fetc+TRAIN_FIELDS,
-                                  definedVocabulary, vocabulary) || !storeTranscriptionAndFields(adaptionType, promptsPathIn,
-                                  fetc+TEST_TRANSCRIPTION, fetc+TEST_FIELDS, definedVocabulary, vocabulary))
+  if(!(adaptionType == ModelCompilationAdapter::AdaptLanguageModel) &&
+     (!storeTranscriptionAndFields(adaptionType, promptsPathIn, fetc+TRAIN_TRANSCRIPTION, fetc+TRAIN_FIELDS,
+         definedVocabulary, vocabulary)
+      || !storeTranscriptionAndFields(adaptionType, promptsPathIn,
+         fetc+TEST_TRANSCRIPTION, fetc+TEST_FIELDS, definedVocabulary, vocabulary)))
   {
     emit error(i18n("Failed to store transcription and fields"));
     return false;
@@ -170,6 +168,19 @@ bool ModelCompilationAdapterSPHINX::storeModel(AdaptionType adaptionType, const
   return true;
 }
 
+bool ModelCompilationAdapterSPHINX::purgeUnusedVocabulary(QSharedPointer<Vocabulary> vocabulary, QSharedPointer<Grammar> grammar)
+{
+  QSet<QString> terminals;
+  foreach (const QString& structure, grammar->getStructures())
+    foreach (const QString& terminal, structure.split(" ", QString::SkipEmptyParts))
+      terminals.insert(terminal);
+  QList<Word*> words = vocabulary->getWords();
+  foreach (Word* w, words)
+    if (!terminals.contains(w->getTerminal()))
+      vocabulary->removeWord(w);
+  return true;
+}
+
 bool ModelCompilationAdapterSPHINX::storeDictionary(AdaptionType adaptionType, const QString &dictionaryPathOut, QStringList &trainedVocabulary,
                                                     QStringList &definedVocabulary, QSharedPointer<Vocabulary> vocabulary)
 {
@@ -190,9 +201,9 @@ bool ModelCompilationAdapterSPHINX::storeDictionary(AdaptionType adaptionType, c
   m_wordCount = 0;
   foreach (Word *word, words)
   {
-    if (//(adaptionType & ModelCompilationAdapter::AdaptAcousticModel) &&
+    if ((adaptionType & ModelCompilationAdapter::AdaptAcousticModel) &&
             (!(adaptionType == ModelCompilationAdapter::AdaptIndependently) &&
-        !trainedVocabulary.contains(word->getLexiconWord())) || added.contains(word->getWord()))
+        (!trainedVocabulary.contains(word->getLexiconWord()) || added.contains(word->getWord()))))
     {
       kDebug() << "Skipping word " << word->getWord();
       continue;
@@ -205,7 +216,7 @@ bool ModelCompilationAdapterSPHINX::storeDictionary(AdaptionType adaptionType, c
     added.append(word->getWord());
 
     ++m_wordCount;
-    if (//(adaptionType & ModelCompilationAdapter::AdaptAcousticModel) &&
+    if (//(adaptionType == ModelCompilationAdapter::AdaptAcousticModel) &&
         (!definedVocabulary.contains(word->getLexiconWord())))
       definedVocabulary << word->getLexiconWord();
   }
@@ -217,6 +228,7 @@ bool ModelCompilationAdapterSPHINX::storeDictionary(AdaptionType adaptionType, c
 
 bool ModelCompilationAdapterSPHINX::storeFiller(AdaptionType adaptionType, const QString &fillerPathOut)
 {
+  Q_UNUSED(adaptionType);
   QFile fillerFile(fillerPathOut);
   if (!fillerFile.open(QIODevice::WriteOnly))
   {
@@ -252,7 +264,8 @@ bool ModelCompilationAdapterSPHINX::storePhonesList(AdaptionType adaptionType, c
   QSet<QString> uniquePhonemes;
   foreach (Word *word, vocabulary->getWords())
   {
-    if((adaptionType != AdaptIndependently) && !trainedVocabulary.contains(word->getLexiconWord()))
+    if((adaptionType & ModelCompilationAdapter::AdaptAcousticModel) &&
+       (adaptionType != AdaptIndependently) && !trainedVocabulary.contains(word->getLexiconWord()))
     {
       kDebug() << "Skipping phones for word " << word->getWord();
       continue;
@@ -320,9 +333,8 @@ bool ModelCompilationAdapterSPHINX::storeTranscriptionAndFields(AdaptionType ada
         wordsString.append(word.toLower());//WARNING: error or not?
     }
 
-    if (allWordsInLexicon || adaptionType == AdaptIndependently)
+    if (allWordsInLexicon || !(adaptionType & AdaptLanguageModel) || (adaptionType & AdaptIndependently))
     {
-
       promptsOutFile.write("<s> "+wordsString.toUtf8() + " </s> (" + line.left(splitter).toUtf8() /*filename*/+ ")\n");
       fieldsOutFile.write(line.left(splitter).toUtf8() /*filename*/ + "\n");
       ++m_sampleCount;
@@ -355,6 +367,9 @@ bool ModelCompilationAdapterSPHINX::storeGrammar(ModelCompilationAdapter::Adapti
   grammarStream<< "public <structure> = ";
 
   QStringList grammarStructures = grammar->getStructures();
+
+  kDebug()<<"Structures count:"<< grammarStructures.size();
+
   foreach (const QString& structure, grammarStructures)
   {
     QStringList terminals = Grammar::getTerminalsForStructure(structure);
@@ -368,11 +383,14 @@ bool ModelCompilationAdapterSPHINX::storeGrammar(ModelCompilationAdapter::Adapti
 
     QString gramBuffer;
 
+    kDebug()<<"Terminals count for structure "<<structure<<": "<<terminals.size();
     foreach (const QString &terminal, terminals)
     {
       bool fword = true;
       QList<Word*> wordsForTerminal = vocabulary->findWordsByTerminal(terminal);
 
+      kDebug()<<"Words for terminal "<<terminal<<":"<<wordsForTerminal.size();
+
       if(wordsForTerminal.isEmpty())
         continue;
 
@@ -381,7 +399,7 @@ bool ModelCompilationAdapterSPHINX::storeGrammar(ModelCompilationAdapter::Adapti
 
       foreach (Word* word, wordsForTerminal)
       {
-        if(!definedVocabulary.contains(word->getLexiconWord()) && adaptionType != AdaptIndependently)
+        if(!definedVocabulary.contains(word->getLexiconWord()) && adaptionType != AdaptIndependently)//WARNING: talk about adapt independently becouse of magic
           continue;
 
         if(!fword)
