@@ -20,6 +20,7 @@
 #include "modelcompilationadaptersphinx.h"
 #include "modelcompilersphinx.h"
 #include "modelcompilationmanagersphinx.h"
+#include <simonutils/fileutils.h>
 #include <KStandardDirs>
 #include <QFileInfo>
 #include <QUuid>
@@ -42,7 +43,7 @@ ModelCompilationManagerSPHINX::ModelCompilationManagerSPHINX(const QString& user
 
   connect(compiler, SIGNAL(wordUndefined(QString)), this, SIGNAL(wordUndefined(QString)));
   connect(compiler, SIGNAL(classUndefined(QString)), this, SIGNAL(classUndefined(QString)));
-  connect(compiler, SIGNAL(phonemeUndefined(QString)), this, SIGNAL(phonemeUndefined(QString)));
+  connect(compiler, SIGNAL(phonemeUndefined(QString)), this, SLOT(slotPhonemeUndefined(QString)));
 }
 
 void ModelCompilationManagerSPHINX::run()
@@ -50,9 +51,10 @@ void ModelCompilationManagerSPHINX::run()
   //first, adapt the input to sphinx readable formats using the adapter
   QHash<QString,QString> adaptionArgs;
 
-  QString activeDir = KStandardDirs::locateLocal("appdata", "models/"+userName+"/active/");
+//  QString activeDir = KStandardDirs::locateLocal("appdata", "models/"+userName+"/active/");
   QUuid modelUuid = QUuid::createUuid();
 
+  kDebug() <<"baseModelType: "<<baseModelType;
   ModelCompilationAdapter::AdaptionType adaptionType = (baseModelType == 0) ?
                                                          (ModelCompilationAdapter::AdaptLanguageModel) :
                                                          (ModelCompilationAdapter::AdaptionType) (ModelCompilationAdapter::AdaptAcousticModel|ModelCompilationAdapter::AdaptLanguageModel);
@@ -76,6 +78,19 @@ void ModelCompilationManagerSPHINX::run()
   {
     if (!keepGoing) return;
 
+    QString baseModelFolder;
+    if (baseModelType < 2)
+    {
+      baseModelFolder = KStandardDirs::locateLocal("tmp", KGlobal::mainComponent().aboutData()->appName()+'/'+userName+"/compile/base/");
+      //base model needed - unpack it and fail if its not here
+      if (!FileUtils::unpackAll(baseModelPath, baseModelFolder))
+      {
+        emit error(i18nc("%1 is path to the base model", "Could not open base model at \"%1\".", baseModelPath));
+        return;
+      }
+      compilerArgs.insert("baseModelDir", baseModelFolder);
+    }
+
     tryAgain = false;
     if (!adapter->startAdaption(adaptionType, scenarioPaths, promptsPathIn, adaptionArgs))
     {
@@ -99,29 +114,45 @@ void ModelCompilationManagerSPHINX::run()
       return;
     }
 
+    if (adapter->sampleCount() == 0)
+    {
+      switch (baseModelType)
+      {
+        case 1:
+          kDebug() << "No Prompts!  Switching to static model!";
+          baseModelType = 0;
+          break;
+        case 2:                                     //do not bother creating the model without prompts
+          kDebug() << "No Prompts!  Model recompilation aborting!";
+          emit modelCompilationAborted(ModelCompilation::InsufficientInput);
+          return;
+      }
+    }
     ModelCompiler::CompilationType compilationType = getCompilationType(baseModelType);
 
 
     //build fingerprint and search cache for it
     uint fingerprint = 0;
-    QStringList componentsToParse(QStringList() << GRAMMAR_EXT << PHONE_EXT << DICT_EXT);
+    QStringList componentsToParse(QStringList() << fetc + GRAMMAR_EXT << fetc + DICT_EXT);
     if (baseModelType > 0)
-      componentsToParse << TRAIN_TRANSCRIPTION << TRAIN_FIELDS ;
+      componentsToParse << fetc + TRAIN_TRANSCRIPTION << fetc + TRAIN_FIELDS << fetc + PHONE_EXT;
+    if (baseModelType < 2)
+      componentsToParse << baseModelPath;
 
-    fingerprint = getFingerPrint(fetc, componentsToParse, compilationType);
+    fingerprint = getFingerPrint(componentsToParse, compilationType);
 
     bool exists;
     QString outPath = cachedModelPath(fingerprint, &exists);
 
     if (!keepGoing) return;
 
-    if (exists || compiler->startCompilation(compilationType, outPath, adapter->getDroppedTranscriptions(), 
-                                             activeDir, compilerArgs))
+    if (exists || compiler->startCompilation(compilationType, outPath, adapter->getDroppedTranscriptions(),
+                                             baseModelPath, compilerArgs))
     {
       emit modelReady(fingerprint, outPath);
       return;
     } else
       kWarning() << "Model compilation failed for user " << userName;
   } while (tryAgain);
-  emit modelCompilationAborted(ModelCompilation::InsufficientInput);
+  keepGoing = false;
 }
